@@ -9,6 +9,7 @@ package inc::MBX::FLTK;
     use File::Find qw[find];
     use File::Path qw[make_path];
     use base 'Module::Build';
+    our %dl_funcs;
     {
 
         package My::ExtUtils::CBuilder;
@@ -26,47 +27,114 @@ package inc::MBX::FLTK;
             #(my( $program), @cmd) = @cmd;
             #return !system ($program, @cmd);
         }
+
+        sub prelink {
+            my ($self, %args) = @_;
+            ($args{dl_file} = $args{dl_name}) =~ s/.*:://
+                unless $args{dl_file};
+            $args{'dl_funcs'} = \%inc::MBX::FLTK::dl_funcs; # Frakin' CBuilder
+            require ExtUtils::Mksymlists;
+            ExtUtils::Mksymlists::Mksymlists( # dl. abbrev for dynamic library
+                DL_VARS  => $args{dl_vars}      || [],
+                DL_FUNCS => $args{dl_funcs}     || {},
+                FUNCLIST => $args{dl_func_list} || [],
+                IMPORTS  => $args{dl_imports}   || {},
+                NAME   => $args{dl_name},     # Name of the Perl module
+                DLBASE => $args{dl_base},     # Basename of DLL file
+                FILE   => $args{dl_file},     # Dir + Basename of symlist file
+                VERSION =>
+                    (defined $args{dl_version} ? $args{dl_version} : '0.0'),
+            );
+
+            # Mksymlists will create one of these files
+            return grep -e, map "$args{dl_file}.$_", qw(ext def opt);
+        }
     }
     {
 
         sub ACTION_code {
-            require Alien::FLTK2;    # Should be installed by now
+            require Alien::FLTK2;             # Should be installed by now
             my ($self, $args) = @_;
             my $AF = Alien::FLTK2->new();
             my $CC = My::ExtUtils::CBuilder->new();
-            my (@xs, @rc, @obj);
+            my (@xs, @rc, @pl, @obj);
             find(sub { push @xs, $File::Find::name if m[.+\.xs$]; }, 'xs');
-            find(sub { push @rc, $File::Find::name if !m[.+\.o$]; }, 'xs/rc');
+            find(sub { push @pl, $File::Find::name if m[.+\.pl$]i; },
+                 'xs/rc');
             if ($self->is_windowsish) {
+                $self->do_system($^X, $_) for @pl;
+                find(sub { push @rc, $File::Find::name if m[.+\.rc$]; },
+                     'xs/rc');
                 my @dot_rc = grep defined,
                     map { m[\.(rc)$] ? rel2abs($_) : () } @rc;
                 for my $dot_rc (@dot_rc) {
-                    my $dot_o
-                        = $dot_rc =~ m[^(.*)\.] ? $1 . $Config{'_o'} : next;
+                    my $dot_o = $dot_rc =~ m[^(.*)\.] ? $1 . '.res' : next;
                     push @obj, $dot_o;
                     next if $self->up_to_date($dot_rc, $dot_o);
                     printf 'Building Win32 resource: %s... ',
                         abs2rel($dot_rc);
-                    chdir 'xs/rc';
-                    print $self->do_system(sprintf 'windres %s %s',
+                    chdir $self->base_dir . '/xs/rc';
+                    require Config;
+                    my $cc = $Config{'ccname'} || $Config{'cc'};
+                    if ($cc eq 'cl') {    # MSVC
+                        print $self->do_system(
+                                      sprintf 'rc.exe /l 0x409 /fo"%s" %s',
+                                      $dot_o, $dot_rc) ? "okay\n" : "fail!\n";
+                    }
+                    else {                # GCC
+                        print $self->do_system(
+                                      sprintf 'windres -O coff -i %s -o %s',
                                       $dot_rc, $dot_o) ? "okay\n" : "fail!\n";
+                    }
                     chdir rel2abs($self->base_dir);
                 }
-                map { abs2rel($_) } @obj;
+                map { abs2rel($_) if -f } @obj;
             }
-        XS: for my $XS (@xs) {
-                my $cpp = _xs_to_cpp($self, $XS)
-                    or do { printf 'Cannot Parse %s', $XS; exit 0 };
+            my @cpp;
+        XS: for my $XS ((sort { lc $a cmp lc $b } @xs)) {
+                push @cpp, _xs_to_cpp($self, $XS)
+                    or exit !printf 'Cannot Parse %s', $XS;
+                {
+                    open(my ($FH), '<', $XS) || return;
+                    sysread($FH, my ($content), -s $FH) == -s $FH or return;
+                    my @packages = $content =~ m[^MODULE\s*?=\s*?(\S+).+?$]mg;
+                    close $FH;
+                    for my $package (@packages) { $dl_funcs{$package} = []; }
+                }
+            }
+            if (!$self->up_to_date(\@xs, 'xs/include/FLTK_pm_boot.h')) {
+
+                # Generate xs/includes/boot.h
+                print q[Generating 'xs/include/FLTK_pm_boot.h'... ];
+                open(my ($BOOT_H), '>', 'xs/include/FLTK_pm_boot.h')
+                    || die 'Failed to create boot header: ' . $!;
+                my @lines = sprintf "/* Autogenerated by %s:%d at %s */;",
+                    __FILE__, __LINE__, scalar gmtime;
+                push @lines, 'void reboot() {';
+                push @lines, qq[    call( cvrv, "$_" );]
+                    for sort { uc $a cmp uc $b }
+                    grep     {m[^FLTK::]} keys %dl_funcs;
+                push @lines, '}';
+                syswrite($BOOT_H, join "\r\n", @lines);
+                close $BOOT_H;
+                $self->add_to_cleanup('xs/include/FLTK_pm_boot.h');
+                print "okay\n";
+            }
+        CPP: for my $cpp (@cpp) {
                 if ($self->up_to_date($cpp, $CC->object_file($cpp))) {
                     push @obj, $CC->object_file($cpp);
-                    next XS;
+                    next CPP;
                 }
-                push @obj,
-                    $CC->compile('C++'        => 1,
-                                 source       => $cpp,
-                                 include_dirs => [$AF->include_dirs()],
-                                 extra_compiler_flags => [$AF->cxxflags()]
-                    );
+                local $CC->{'quiet'} = 1;
+                printf q[Building '%s'... ], $cpp;
+                my $obj = $CC->compile(
+                                     'C++'        => 1,
+                                     source       => $cpp,
+                                     include_dirs => [$AF->include_dirs()],
+                                     extra_compiler_flags => [$AF->cxxflags()]
+                );
+                printf "%s\n", $obj ? 'okay' : 'failed';    # XXX - exit?
+                push @obj, $obj;
             }
             make_path(catdir(qw[blib arch auto FLTK]),
                       {verbose => !$self->quiet(), mode => 0777});
@@ -79,13 +147,14 @@ package inc::MBX::FLTK;
                 )
             {   my ($dll, @cleanup)
                     = $CC->link(
-                          objects => \@obj,
-                          lib_file =>
-                              catdir(qw[blib arch auto FLTK],
-                                     'FLTK.' . $Config{'so'}
-                              ),
-                          module_name        => 'FLTK',
-                          extra_linker_flags => [$AF->ldflags(qw[gl images])],
+                              objects => \@obj,
+                              lib_file =>
+                                  catdir(qw[blib arch auto FLTK],
+                                         'FLTK.' . $Config{'so'}
+                                  ),
+                              module_name => 'FLTK',
+                              extra_linker_flags =>
+                                  [$AF->ldflags('images', $AF->capabilities)],
                     );
                 @cleanup = map { s["][]g; rel2abs($_); } @cleanup;
                 $self->add_to_cleanup(@cleanup);
@@ -105,16 +174,22 @@ package inc::MBX::FLTK;
                 catdir('xs');
             $self->add_to_cleanup($cpp);
             return $cpp
-                if $self->up_to_date([@xsi, $xs, catdir('xs', $typemap)],
-                                     $cpp);
-            printf '%s -> %s (%s)... ', $xs, $cpp, $typemap;
+                if $self->up_to_date([@xsi, $xs,
+                                      rel2abs(catdir('xs', $typemap))
+                                     ],
+                                     $cpp
+                );
+            printf q"Parsing '%s' into '%s' w/ '%s'... ", $xs, $cpp, $typemap;
+            local @ExtUtils::ParseXS::BootCode = ();
 
-            if (ExtUtils::ParseXS->process_file(filename   => $xs,
-                                                output     => $cpp,
-                                                'C++'      => 1,
-                                                hiertype   => 1,
-                                                typemap    => $typemap,
-                                                prototypes => 1
+            if (ExtUtils::ParseXS->process_file(
+                                   filename => $xs,
+                                   output   => $cpp,
+                                   'C++'    => 1,
+                                   hiertype => 1,
+                                   typemap => rel2abs(catdir('xs', $typemap)),
+                                   prototypes  => 1,
+                                   linenumbers => 1
                 )
                 )
             {   print "okay\n";
@@ -141,10 +216,9 @@ package inc::MBX::FLTK;
         {
 
             package Pod::APIDoc::FLTK;
-            use Pod::Parser;
+            use base 'Pod::Parser';
             use strict;
             use warnings;
-            our @ISA = qw[Pod::Parser];
             my ($package, $function, $current) = ('FLTK');
 
             sub command {
@@ -215,8 +289,9 @@ package inc::MBX::FLTK;
                 if ($line =~ m[^INCLUDE: (.+\.xsi?)]) {
                     $self->parse_from_file('xs/' . $1);
                 }
-                elsif ($line =~ m[^MODULE\s*=\s*FLTK\s+PACKAGE\s*=\s*(.+)]) {
-                    $package = $1;
+                elsif ($line
+                      =~ m[^MODULE\s*=\s*FLTK(?:::\w+)?\s+PACKAGE\s*=\s*(.+)])
+                {   $package = $1;
                 }
                 return $line;
             }
@@ -240,9 +315,14 @@ package inc::MBX::FLTK;
             my ($self) = @_;
             my $parser = Pod::APIDoc::FLTK->new();
             $parser->parseopts(-want_nonPODs => 1);
-            print 'Parsing XS files for documentation... ';
-            $parser->parse_from_file('xs/FLTK.xs');
-            print "okay\nGenerating documentation... ";
+            my @xs;
+            find(sub { push @xs, $File::Find::name if m[.+\.xs$]; }, 'xs');
+            for my $xs (sort { lc $a cmp lc $b } @xs) {
+                printf q[Parsing '%s' for documentation... ], $xs;
+                $parser->parse_from_file($xs);
+                print "\n";
+            }
+            print "Generating documentation... ";
             for my $package (sort keys %{$parser->{'apidoc_modules'}}) {
                 my $file = './blib/lib/' . $package . '.pod';
                 $file =~ s|::|/|g;
@@ -417,15 +497,15 @@ package inc::MBX::FLTK;
 
 =pod
 
-=for $Rev: 7aee1fa $
+=for $Rev: a42720f $
 
-=for $Revision: 7aee1fa6f73b4e2bfb8a9a5e8470ec4c0aba114f $
+=for $Revision: a42720f2664562bb3248006d107f257f3093c432 $
 
-=for $Date: 2010-04-12 14:30:14Z (Mon, 12 Apr 2010) $ | Last $Modified: 5 days ago $
+=for $Date: 2010-09-25 05:55:13Z (Sat, 25 Sep 2010) $ | Last $Modified: 2 days ago $
 
 =for $URL: http://github.com/sanko/perl-fltk2/raw/master/inc/MBX/FLTK.pm $
 
-=for $ID: FLTK.pm 7aee1fa 2010-04-12 14:30:14Z sanko@cpan.org $
+=for $ID: FLTK.pm a42720f 2010-09-25 05:55:13Z sanko@cpan.org $
 
 =for author Sanko Robinson <sanko@cpan.org> - http://sankorobinson.com/
 
@@ -433,15 +513,18 @@ package inc::MBX::FLTK;
 
 sub LICENSE {
     <<'ARTISTIC_TWO' }
-Copyright (C) 2008-2009 by Sanko Robinson E<lt>sanko@cpan.orgE<gt>
+Copyright (C) 2008-2010 by Sanko Robinson <sanko@cpan.org>
 
 This program is free software; you can redistribute it and/or modify it under
-the terms of The Artistic License 2.0.  See the F<LICENSE> file included with
-this distribution or http://www.perlfoundation.org/artistic_license_2_0.  For
-clarification, see http://www.perlfoundation.org/artistic_2_0_notes.
+the terms of
+L<The Artistic License 2.0|http://www.perlfoundation.org/artistic_license_2_0>.
+See the F<LICENSE> file included with this distribution or
+L<notes on the Artistic License 2.0|http://www.perlfoundation.org/artistic_2_0_notes>
+for clarification.
 
 When separated from the distribution, all original POD documentation is
-covered by the Creative Commons Attribution-Share Alike 3.0 License.  See
-http://creativecommons.org/licenses/by-sa/3.0/us/legalcode.  For
-clarification, see http://creativecommons.org/licenses/by-sa/3.0/us/.
+covered by the
+L<Creative Commons Attribution-Share Alike 3.0 License|http://creativecommons.org/licenses/by-sa/3.0/us/legalcode>.
+See the
+L<clarification of the CCA-SA3.0|http://creativecommons.org/licenses/by-sa/3.0/us/>.
 ARTISTIC_TWO
